/**
 * Copy one file to another.
 *
 * Usage:
 *     copy <from> <to>
 */

#include <stdio.h>
#include <stdbool.h>
#include <stdlib.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#ifndef O_BINARY
#define O_BINARY 0	// Define O_BINARY if not defined
#endif // O_BINARY

int main(int argc, char* argv[])
{
    if (argc != 3) {
	fprintf(stderr, "Usage is %s <infile> <outfile>\n", argv[0]);
	exit(8);
    }

    // The fd of the input file
    int inFd = open(argv[1], O_RDONLY|O_BINARY);

    if (inFd < 0) {
	fprintf(stderr, "ERROR: Could not open %s for input\n", argv[1]);
	exit(8);
    }

    // The fd of the output file
    int outFd = open(argv[2], O_WRONLY|O_CREAT|O_BINARY, 0666);
    if (outFd < 0) {
	fprintf(stderr, "ERROR: Could not open %s for writing\n", argv[2]);
	exit(8);
    }

    while (true)
    {
	char buffer[1024];	// Buffer to read and write
	ssize_t readSize;	// Size of the last read

	readSize = read(inFd, buffer, sizeof(buffer));
	if (readSize < 0) {
	    fprintf(stderr, "ERROR: Read error for file %s\n", argv[1]);
	    exit(8);
	}
	if (readSize == 0)
	    break;

	if (write(outFd, buffer, readSize) != readSize) {
	    fprintf(stderr, "ERROR: Write error for %s\n", argv[2]);
	    exit(8);
	}
    }
    close(inFd);
    close(outFd);
    return (0);
}

